@@ -223,6 +223,11 @@
 #define DRBD_CSUMS_AFTER_CRASH_ONLY_DEF 0
 #define DRBD_AUTO_PROMOTE_DEF	1
 
+#define DRBD_NR_REQUESTS_MIN	4
+#define DRBD_NR_REQUESTS_DEF	8000
+#define DRBD_NR_REQUESTS_MAX	-1U
+#define DRBD_NR_REQUESTS_SCALE	'1'
+
 #define DRBD_MAX_BIO_SIZE_DEF	DRBD_MAX_BIO_SIZE
 #define DRBD_MAX_BIO_SIZE_MIN	(1 << 9)
 #define DRBD_MAX_BIO_SIZE_MAX	DRBD_MAX_BIO_SIZE
